/*
 *   SPDX-FileCopyrightText: 2010 Casey Link <unnamedrambler@gmail.com>
 *   SPDX-FileCopyrightText: 2009-2010 Klaralvdalens Datakonsult AB, a KDAB Group company <info@kdab.net>
 *
 *   SPDX-License-Identifier: GPL-2.0-or-later
 *
 */

#pragma once

/* contains one maildir with one  email inside*/
static const char mailDirOneEmail[] =
    "IyBLTWFpbC1JbmRleCBWMTUwNgoACAAAAHhWNBIIAAAAagEAAAUAAAAAAAgAAAAIAAAAAAAAAAAA"
    "AQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAAAACAAAAIAAAaABlAGwAbABvACAAZgByAG8A"
    "bQAgAGsAbQBhAGkAbAMAAAAcAABLAEQAQQBCACAARQBtAHAAbABvAHkAZQBlAHMEAAAAAAAGAAAA"
    "AAALAAAANAAAMQAyADcAMQA1ADEAMwA1ADMANwAuADEAOQA1ADcANgAuADEATABLADYAZQA6ADIA"
    "LABTCQAAAAgAhAIAAAAAAAAHAAAACAAAAAAAAAAAAAoAAAAIAMHByUsAAAAADAAAAAgATgBOAAAA"
    "AAANAAAACAAgAAAAAAAAAA4AAAAAAA8AAAAsAABxADEAUABJAFkAbgBhAGwAVwB3AFYAQgAzAHEA"
    "aAB2AEYAUABUAHQAaQBBEAAAAAgABAAAAAAAAAARAAAACAAAAAAAAAAAABIAAAAIAAAAAAAAAAAA";

/*
 Folder: "Test1" type:Maildir
 Message1: "foo bar"  Important, Unread
 Message2: "hello from kmail" Read, tagged "N5tUHPOZFf"
 */
static const char mailDirTwoEmailOneTagFlags[] =
    "IyBLTWFpbC1JbmRleCBWMTUwNgoACAAAAHhWNBIIAAAAlAEAAAUAAAAAAAgAAAAIAAAAAAAAAAAA"
    "AQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAKgAATgA1AHQAVQBIAFAATwBaAEYAZgAsAFUA"
    "RgBNAEUAagBHAHMAUQA5AG8CAAAAIAAAaABlAGwAbABvACAAZgByAG8AbQAgAGsAbQBhAGkAbAMA"
    "AAAcAABLAEQAQQBCACAARQBtAHAAbABvAHkAZQBlAHMEAAAAAAAGAAAAAAALAAAANAAAMQAyADcA"
    "MQA1ADEAMwA1ADMANwAuADEAOQA1ADcANgAuADEATABLADYAZQA6ADIALABTCQAAAAgAhAIAAAAA"
    "AAAHAAAACAAAAAAAAAAAAAoAAAAIAMHByUsAAAAADAAAAAgATgBOAAAAAAANAAAACAAgAAAAAAAA"
    "AA4AAAAAAA8AAAAsAABxADEAUABJAFkAbgBhAGwAVwB3AFYAQgAzAHEAaAB2AEYAUABUAHQAaQBB"
    "EAAAAAgABAAAAAAAAAARAAAACAAAAAAAAAAAABIAAAAIAAAAAAAAAAAASAEAAAUAAAAAAAgAAAAI"
    "AAAAAAAAAAAAAQAAABQAAEMAYQBzAGUAeQAgAEwAaQBuAGsTAAAAAAACAAAADgAAZgBvAG8AIABi"
    "AGEAcgMAAAAWAABvAG4AZQBAAG8AbgBlAC4AYwBvAG0EAAAAAAAGAAAAAAALAAAAKgAAMQAyADcA"
    "MQA3ADcAMwAwADEAOQAuADUANwA1ADQALgBwAFcAcQBaAFMJAAAACABKAgAAAAAAAAcAAAAIAAAA"
    "AAAAAAAACgAAAAgAW7fNSwAAAAAMAAAACABOAE4AAAAAAA0AAAAIACAAAAAAAAAADgAAAAAADwAA"
    "ACwAAE0AbgB0AHYAQgAwAE4AWQBFAFMATwBiAHgASAA0AFYAUgBEAFUAeQBjAHcQAAAACAACAgAA"
    "AAAAABEAAAAIAAAAAAAAAAAAEgAAAAgAAAAAAAAAAAA=";
