/*
 * SPDX-FileCopyrightText: 2010 Klaralvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 * SPDX-FileCopyrightText: 2010 Leo Franchi <lfranchi@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#include "imapflags.h"

const char *ImapFlags::Seen = "\\Seen";
const char *ImapFlags::Deleted = "\\Deleted";
const char *ImapFlags::Answered = "\\Answered";
const char *ImapFlags::Flagged = "\\Flagged";
