/*
   SPDX-FileCopyrightText: 2011 Daniel Vrátil <dvratil@redhat.com>
   SPDX-FileCopyrightText: 2023 Carl Schwan <carl@carlschwan.eu>
   SPDX-License-Identifier: GPL-3.0-or-later
*/

#include <QList>
#include <QUrl>

QList<QUrl> googleScopes();
