/*
    SPDX-FileCopyrightText: 2018 Daniel Vrátil <dvratil@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "maildirconfig.h"

AKONADI_AGENTCONFIG_FACTORY(MaildirConfigFactory, "maildirconfig.json", MaildirConfig)

#include "maildirconfigmain.moc"
